@@ -1,6 +1,6 @@
 use std::{convert::Infallible, future, net::ToSocketAddrs};
 
-use tower::{Layer, Service};
+use tower::{Layer, Service as TowerService};
 
 use crate::{HttpRequest, HttpResponse};
 use handler::Handler;
@@ -33,11 +33,18 @@ pub mod gen_prelude {
         router::Routes,
         socket::Socket,
         utils::serve,
-        Server,
+        Service,
     };
-    pub use crate::{body::box_body, Request as HrpcRequest, Response as HrpcResponse};
-    pub use crate::{HttpRequest, HttpResponse};
-    pub use tower::{layer::util::Identity, Layer};
+    pub use crate::{
+        body::box_body, BoxError, HttpRequest, HttpResponse, Request as HrpcRequest,
+        Response as HrpcResponse,
+    };
+    pub use bytes::Bytes;
+    pub use futures_util::future::BoxFuture;
+    pub use http::Response as _HttpResponse;
+    pub use http_body::Body as HttpBody;
+    pub use std::{convert::Infallible, future::Future, sync::Arc};
+    pub use tower::{layer::util::Identity, Layer, Service as TowerService};
 }
 
 /// Prelude that exports commonly used server types.
@@ -46,7 +53,7 @@ pub mod prelude {
         error::{CustomError, ServerResult},
         handler::HrpcLayer,
         socket::Socket,
-        Server,
+        Service,
     };
     pub use crate::{IntoResponse, Request, Response};
     pub use async_trait::async_trait;
@@ -54,19 +61,19 @@ pub mod prelude {
 }
 
 /// The core trait of `hrpc-rs` servers. This trait acts as a `tower::MakeService`,
-/// it can produce a set of [`Routes`] and can be combined with other [`Server`]s.
+/// it can produce a set of [`Routes`] and can be combined with other [`Service`]s.
 #[async_trait::async_trait]
-pub trait Server: Send + 'static {
+pub trait Service: Send + 'static {
     /// Creates a [`Routes`], which will be used to build a [`RoutesFinalized`] instance.
     fn make_routes(&self) -> Routes;
 
     /// Combines this server with another server.
-    fn combine_with<Other>(self, other: Other) -> ServerStack<Other, Self>
+    fn combine_with<Other>(self, other: Other) -> ServiceStack<Other, Self>
     where
-        Other: Server,
+        Other: Service,
         Self: Sized,
     {
-        ServerStack {
+        ServiceStack {
             outer: other,
             inner: self,
         }
@@ -81,18 +88,19 @@ pub trait Server: Send + 'static {
         IntoMakeService { mk_router: self }
     }
 
-    /// Layers this server with a layer producer that produces layers,
-    /// and those layers will be used to transforms handlers.
-    fn layer<F, L>(self, f: F) -> LayeredServer<F, L, Self>
+    /// Layers this server with a layer.
+    ///
+    /// If your layer does not implement [`Clone`], you can wrap it using
+    /// [`HrpcLayer::new`].
+    fn layer<L>(self, layer: L) -> LayeredService<L, Self>
     where
-        F: Fn() -> L + 'static,
-        L: Layer<Handler, Service = Handler> + Send + 'static,
+        L: Layer<Handler, Service = Handler> + Clone + Sync + Send + 'static,
         Self: Sized,
     {
-        LayeredServer { inner: self, f }
+        LayeredService { inner: self, layer }
     }
 
-    /// Serves this server. See [`utils::serve`] for more information.
+    /// Serves this service. See [`utils::serve`] for more information.
     async fn serve<A>(self, address: A) -> Result<(), hyper::Error>
     where
         A: ToSocketAddrs + Send,
@@ -103,57 +111,54 @@ pub trait Server: Send + 'static {
     }
 }
 
-/// Type that layers the handlers that are produced by a [`Server`].
-pub struct LayeredServer<F, L, S>
+/// Type that layers the handlers that are produced by a [`Service`].
+pub struct LayeredService<L, S>
 where
-    F: Fn() -> L + 'static,
-    L: Layer<Handler, Service = Handler> + Send + 'static,
-    S: Server,
+    L: Layer<Handler, Service = Handler> + Clone + Sync + Send + 'static,
+    S: Service,
 {
     inner: S,
-    f: F,
+    layer: L,
 }
 
-impl<F, L, S> Clone for LayeredServer<F, L, S>
+impl<L, S> Clone for LayeredService<L, S>
 where
-    F: Fn() -> L + 'static + Clone,
-    L: Layer<Handler, Service = Handler> + Send + 'static,
-    S: Server + Clone,
+    L: Layer<Handler, Service = Handler> + Clone + Sync + Send + 'static,
+    S: Service + Clone,
 {
     fn clone(&self) -> Self {
         Self {
             inner: self.inner.clone(),
-            f: self.f.clone(),
+            layer: self.layer.clone(),
         }
     }
 }
 
-impl<F, L, S> Server for LayeredServer<F, L, S>
+impl<L, S> Service for LayeredService<L, S>
 where
-    F: Fn() -> L + Send + 'static,
-    L: Layer<Handler, Service = Handler> + Send + 'static,
-    S: Server,
+    L: Layer<Handler, Service = Handler> + Clone + Sync + Send + 'static,
+    S: Service,
 {
     fn make_routes(&self) -> Routes {
-        let rb = Server::make_routes(&self.inner);
-        rb.layer_all((self.f)())
+        let rb = Service::make_routes(&self.inner);
+        rb.layer_all(self.layer.clone())
     }
 }
 
-/// Type that contains two [`Server`]s and stacks (combines) them.
-pub struct ServerStack<Outer, Inner>
+/// Type that contains two [`Service`]s and stacks (combines) them.
+pub struct ServiceStack<Outer, Inner>
 where
-    Outer: Server,
-    Inner: Server,
+    Outer: Service,
+    Inner: Service,
 {
     outer: Outer,
     inner: Inner,
 }
 
-impl<Outer, Inner> Clone for ServerStack<Outer, Inner>
+impl<Outer, Inner> Clone for ServiceStack<Outer, Inner>
 where
-    Outer: Server + Clone,
-    Inner: Server + Clone,
+    Outer: Service + Clone,
+    Inner: Service + Clone,
 {
     fn clone(&self) -> Self {
         Self {
@@ -163,25 +168,25 @@ where
     }
 }
 
-impl<Outer, Inner> Server for ServerStack<Outer, Inner>
+impl<Outer, Inner> Service for ServiceStack<Outer, Inner>
 where
-    Outer: Server,
-    Inner: Server,
+    Outer: Service,
+    Inner: Service,
 {
     fn make_routes(&self) -> Routes {
-        let outer_rb = Server::make_routes(&self.outer);
-        let inner_rb = Server::make_routes(&self.inner);
+        let outer_rb = Service::make_routes(&self.outer);
+        let inner_rb = Service::make_routes(&self.inner);
         outer_rb.combine_with(inner_rb)
     }
 }
 
-/// Type that contains a [`Server`] and implements `tower::MakeService` to
+/// Type that contains a [`Service`] and implements `tower::MakeService` to
 /// create [`RoutesFinalized`] instances.
-pub struct IntoMakeService<S: Server> {
+pub struct IntoMakeService<S: Service> {
     mk_router: S,
 }
 
-impl<S: Server + Clone> Clone for IntoMakeService<S> {
+impl<S: Service + Clone> Clone for IntoMakeService<S> {
     fn clone(&self) -> Self {
         Self {
             mk_router: self.mk_router.clone(),
@@ -189,7 +194,7 @@ impl<S: Server + Clone> Clone for IntoMakeService<S> {
     }
 }
 
-impl<T, S: Server> Service<T> for IntoMakeService<S> {
+impl<T, S: Service> TowerService<T> for IntoMakeService<S> {
     type Response = RoutesFinalized;
 
     type Error = Infallible;
@@ -212,13 +217,13 @@ impl<T, S: Server> Service<T> for IntoMakeService<S> {
 mod tests {
     use tower::layer::util::Identity;
 
-    use crate::server::{router::Routes, HrpcLayer, Server};
+    use crate::server::{router::Routes, HrpcLayer, Service};
 
     use super::utils::recommended_layers;
 
     struct TestServer;
 
-    impl Server for TestServer {
+    impl Service for TestServer {
         fn make_routes(&self) -> Routes {
             Routes::new()
         }
@@ -230,7 +235,7 @@ mod tests {
 
         // we can't poll it, and we don't want to anyways
         let _ = s
-            .layer(|| HrpcLayer::new(Identity::new()))
+            .layer(HrpcLayer::new(Identity::new()))
             .serve("127.0.0.1:2289");
     }
 
@@ -240,7 +245,7 @@ mod tests {
 
         // we can't poll it, and we don't want to anyways
         let _ = s
-            .layer(|| recommended_layers(|_| true))
+            .layer(recommended_layers(|_| true))
             .serve("127.0.0.1:2289");
     }
 }
